"""
This binds four rooms and four shapes.
It then also gets the shape from the room before.
"""

#NEAL files and functions
from nealParams import *

class cogmap:
    fStandAlone = False

    def __init__(self, simName):
        self.simulator_name = simName

    #---helper functions
    def nealProjection(self,preNeurons,postNeurons, connectorList,inhExc):
        if self.simulator_name == 'spiNNaker':
            Projection(preNeurons, postNeurons, connectorList,target=inhExc)
        elif self.simulator_name == 'nest':
            Projection(preNeurons, postNeurons, connectorList)
        else:
            print 'bad simulator for nealProjection'

    #---Stuff for the basic cog map binding
    def allocateRoomNeurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}
        self.roomCells=Population(20,IF_curr_exp,cell_params)

    def allocateShapeFactNeurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}
        self.shapeFactCells=Population(20,IF_curr_exp,cell_params)

    def allocateExternalShapeFactNeurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}
        self.externalShapeFactCells=Population(20,IF_curr_exp,cell_params)


    def makeRoomCAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.roomCells,self.roomCells, excConnector,
                            'excitatory')

    def roomsPrimeEarlierRooms(self):
        synWeight = 0.02
        connector = []
        for fromRoom in range (0,4):
            if (fromRoom == 0):
                toRoom = 3
            else:
                toRoom = fromRoom -1
            for fromNeuron in range (0,5):
                for toNeuron in range (0,5):
                    connector=connector+[(fromNeuron+fromRoom*5,
                                          toNeuron+toRoom*5,synWeight,DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.roomCells,self.roomCells, excConnector,
                            'excitatory')

    #A room prevents both other rooms (not itself or prior) running
    def roomsStopLaterRooms(self):
        synWeight = -2.0
        connector = []
        for toRoomOff in range (1,3):
            for fromRoom in range (0,4):
                toRoom = fromRoom+toRoomOff
                toRoom = toRoom%4
                for fromNeuron in range (0,5):
                    for toNeuron in range (0,5):
                        connector=connector+[(fromNeuron+fromRoom*5,
                                          toNeuron+toRoom*5,synWeight,DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.roomCells,self.roomCells, inhConnector,
                            'inhibitory')

    def makeShapeFactCAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.shapeFactCells,self.shapeFactCells, 
                            excConnector,'excitatory')

    def connectBind(self, bindInput,room,shape):
        #turn on room
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron+room*5,synWeight,DELAY)]
        excConnector = FromListConnector(connector) 
        self.nealProjection(bindInput,self.roomCells,excConnector,'excitatory') 

        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron+shape*5,synWeight,DELAY)]
        excConnector = FromListConnector(connector) 
        self.nealProjection(bindInput,self.shapeFactCells,
                            excConnector,'excitatory') 

    def stopBind(self,stopInput):
        synWeight = -5.0
        connector = []
        for fromNeuron in range (0,5):
            for toNeuron in range (0,20):
                connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        inhConnector = FromListConnector(connector) 
        self.nealProjection(stopInput,self.roomCells,inhConnector,'inhibitory') 

        self.nealProjection(stopInput,self.shapeFactCells,inhConnector,
                            'inhibitory') 

    def testBind(self,testInput,room,shapeFact):
        if (room >= 0):
            synWeight = 20.0
            connector = []
            for toNeuron in range (0,5):
                connector=connector+[(0,toNeuron+room*5,synWeight,DELAY)]
                excConnector = FromListConnector(connector) 
                self.nealProjection(testInput,self.roomCells,
                                    excConnector,'excitatory') 

        if (shapeFact >= 0):
            synWeight = 20.0
            connector = []
            for toNeuron in range (0,5):
                connector=connector+[(0,toNeuron+shapeFact*5,synWeight,DELAY)]
                excConnector = FromListConnector(connector) 
                self.nealProjection(testInput,self.shapeFactCells,
                                    excConnector,'excitatory') 

    def setCogMapBeforeInputs(self):
        #spikeTimes = [[i for i in range(7,20,5)],] 
        spikes = [[70]]
        self.bind1Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 120
        spikes = [[time],[time],[time],[time],[time]]
        self.stop1Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[300]]
        self.bind2Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 350
        spikes = [[time],[time],[time],[time],[time]]
        self.stop2Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[500]]
        self.bind3Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 550
        spikes = [[time],[time],[time],[time],[time]]
        self.stop3Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[700]] #undone if this is 600 650, the room is not fired
        self.bind4Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 750
        spikes = [[time],[time],[time],[time],[time]]
        self.stop4Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[950]]
        self.test1Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 1000
        spikes = [[time],[time],[time],[time],[time]]
        self.stop5Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[1050]]
        self.namedInput=Population(1,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[1000]]
        self.startFSAInput=Population(1,SpikeSourceArray,{'spike_times':spikes})

    def setTestInputBind1(self):
        self.connectBind(self.bind1Input,0,0)
        self.stopBind(self.stop1Input)

        self.connectBind(self.bind2Input,1,1)
        self.stopBind(self.stop2Input)

        self.connectBind(self.bind3Input,2,2)
        self.stopBind(self.stop3Input)

        self.connectBind(self.bind4Input,3,3)
        self.stopBind(self.stop4Input)

        self.testBind(self.test1Input,1,-1)
        self.stopBind(self.stop5Input)

    def setCogMapBindInputs(self):
        #spikeTimes = [[i for i in range(7,20,5)],] 
        spikes = [[57]]
        self.bind1Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 120
        spikes = [[time],[time],[time],[time],[time]]
        self.stop1Input=Population(5,SpikeSourceArray,{'spike_times':spikes})
        spikes = [[350]]
        self.test1Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 500
        spikes = [[time],[time],[time],[time],[time]]
        self.stop2Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[1000]]
        self.bind2Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 1020
        spikes = [[time],[time],[time],[time],[time]]
        self.stop3Input=Population(5,SpikeSourceArray,{'spike_times':spikes})
        spikes = [[1500]]
        self.test2Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 1600
        spikes = [[time],[time],[time],[time],[time]]
        self.stop4Input=Population(5,SpikeSourceArray,{'spike_times':spikes})

        spikes = [[2000]]
        self.bind3Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 2020
        spikes = [[time],[time],[time],[time],[time]]
        self.stop5Input=Population(5,SpikeSourceArray,{'spike_times':spikes})
        spikes = [[2200]]
        self.test3Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 2300
        spikes = [[time],[time],[time],[time],[time]]
        self.stop6Input=Population(5,SpikeSourceArray,{'spike_times':spikes})
        spikes = [[3000]]
        self.bind4Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 3020
        spikes = [[time],[time],[time],[time],[time]]
        self.stop7Input=Population(5,SpikeSourceArray,{'spike_times':spikes})
        spikes = [[3200]]
        self.test4Input=Population(1,SpikeSourceArray,{'spike_times':spikes})
        time = 3300
        spikes = [[time],[time],[time],[time],[time]]
        self.stop8Input=Population(5,SpikeSourceArray,{'spike_times':spikes})


    #set and check four bindings
    def setTestInputBind2(self):
        self.connectBind(self.bind1Input,0,0)
        self.stopBind(self.stop1Input)
        self.testBind(self.test1Input,-1,0)
        self.stopBind(self.stop2Input)

        self.connectBind(self.bind2Input,1,1)
        self.stopBind(self.stop3Input)
        self.testBind(self.test2Input,1,-1)
        self.stopBind(self.stop4Input)

        self.connectBind(self.bind3Input,2,3)
        self.stopBind(self.stop5Input)
        self.testBind(self.test3Input,-1,3)
        self.stopBind(self.stop6Input)

        self.connectBind(self.bind4Input,3,2)
        self.stopBind(self.stop7Input)
        self.testBind(self.test4Input,-1,2)
        self.stopBind(self.stop8Input)


    def connectRoomsAndShapeFacts(self):
        synWeight = 0.2
        connector = []
        for fromNeuron in range (0,20):
            for toNeuron in range (0,20):
                connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        connectorList = FromListConnector(connector) 

        stdp_model = STDPMechanism(
            #timing_dependence = SpikePairRule(),
            timing_dependence = SpikePairRule(tau_plus=16.7, tau_minus=3.7),
            weight_dependence = AdditiveWeightDependence(w_min=0.0, 
                  w_max=2.0, A_plus=0.1, A_minus=0.1)    
        )

        Projection(self.shapeFactCells,self.roomCells,connectorList,
                   synapse_dynamics = SynapseDynamics(slow = stdp_model))

        Projection(self.roomCells,self.shapeFactCells,connectorList,
                   synapse_dynamics = SynapseDynamics(slow = stdp_model))

    def connectExternalFactsToShapeFacts(self):
        synWeight = 5.0
        connector = []
        for connNeuron in range (0,20):
                connector=connector+[(connNeuron,connNeuron,synWeight,DELAY)]
        connectorList = FromListConnector(connector) 

        self.nealProjection(self.externalShapeFactCells,self.shapeFactCells,
                            connectorList,'excitatory') 


    #---Functions for finding room before active 
    def allocateNamedShapeNeurons(self):
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}

        self.namedShapeCells=Population(20,IF_curr_exp,cp)

    numStates = 9
    #2 FSAs GetBefore and Bind
    #GetRoom, FoundRoom, FindSecondRoom, FoundSecondRoom, FoundShape, gotRoom
    #resetBindNets,turnOffBindNets, DoneReset
    def allocateFSANeurons(self):
        #tau_syn_E is changed from default so inhibition works at same speed
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,'tau_syn_E' : 5.0, 
               'tau_syn_I': 5.0,  'v_reset'    : -70.0, 
                       'i_offset' : 0.0}

        self.finiteSACells=Population(self.numStates*5,IF_curr_exp,cp)

    def allocateBeforeShapeNeurons(self):
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}

        self.beforeShapeCells=Population(20,IF_curr_exp,cp)

    def allocateInBeforeRoomNeurons(self):
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
              'i_offset' : 0.0}

        self.inBeforeRoomCells=Population(20,IF_curr_exp,cp)

    def startFSA(self):
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron,synWeight,DELAY)]
        excConnector = FromListConnector(connector) 
        self.nealProjection(self.startFSAInput,self.finiteSACells,
                            excConnector,'excitatory') 
        return

    def makeNamedCAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.namedShapeCells,self.namedShapeCells, 
                            excConnector,'excitatory')

    def makeFSACAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,self.numStates): 
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.finiteSACells, 
                            excConnector,'excitatory')

    def makeBeforeCAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.beforeShapeCells,self.beforeShapeCells, 
                            excConnector,'excitatory')

    def connectNamedToShapeFacts(self):
        synWeight = 2.0
        connector = []
        for cCA in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.namedShapeCells,self.shapeFactCells, 
                            excConnector,'excitatory')


    def statePrimesState(self,fromState,toState):
        synWeight = 0.05
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = fromState*5+fromOff
                        toNeuron = toState*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.finiteSACells, 
                            excConnector,'excitatory')

    def stateStopsState(self,fromState,toState):
        #synWeight = -10.0
        synWeight = -20.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = fromState*5+fromOff
                        toNeuron = toState*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.finiteSACells, 
                            inhConnector,'inhibitory')

    def stateStartsState(self,fromState,toState):
        synWeight = 1.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = fromState*5+fromOff
                        toNeuron = toState*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.finiteSACells, 
                            excConnector,'excitatory')


    def connectFSA(self):
        self.statePrimesState(0,1) #getRoom -> findRoom; use named Shape
        self.stateStopsState(1,0)
        self.stateStartsState(1,2) #findRoom -> findSecondRoom; only rooms on
        self.stateStopsState(2,1)
        self.statePrimesState(2,3) #findSecondRoom -> foundSecondRoom
        self.stateStopsState(3,1)  #rooms can turn state 1 on
        self.stateStopsState(3,2)
        self.statePrimesState(3,4) #foundSecondRoom -> FoundShape
        self.stateStopsState(4,3)
        self.statePrimesState(4,5) #FoundShape -> GotRoom; with beforeShape
        self.stateStopsState(5,4)

        self.stateStartsState(6,7) #resetBindNets -> turnOffBindNets
        self.stateStopsState(7,6)
        self.stateStartsState(7,8) #turnOffBindNets -> nextRoom (shape & room)
        self.stateStopsState(8,7)
        self.stateStopsState(6,8) #resetBindNets -> stops doneReset


    def finiteSATurnsOffNamed(self):
        state = 1 #findRoom
        synWeight = -10.0
        connector = []
        for shape in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                        fromNeuron = state*5+fromOff
                        toNeuron = shape*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.namedShapeCells, 
                            inhConnector,'inhibitory')


    def roomPrimesState(self,room,state,primeWeight):
        synWeight = primeWeight
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = room*5+fromOff
                        toNeuron = state*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.roomCells,self.finiteSACells, 
                            excConnector,'excitatory')
        return

    def statePrimesRoom(self,state,room):
        synWeight = 0.5
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = state*5+fromOff
                        toNeuron = room*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.roomCells, 
                            excConnector,'excitatory')
        return

    def stateStopsRoom(self,state,room):
        synWeight = -20.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = state*5+fromOff
                        toNeuron = room*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.roomCells, 
                            inhConnector,'inhibitory')


    def roomPreventsState(self,room,state):
        synWeight = -10.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = room*5+fromOff
                toNeuron = state*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.roomCells,self.finiteSACells, 
                            inhConnector,'inhibitory')

    def connectRoomsAndFSA(self):
        for room in range (0,4):
            self.roomPrimesState(room,1,0.05)
            self.statePrimesRoom(2,room)
            self.roomPrimesState(room,3,0.05)
            self.stateStopsRoom(4,room)
            self.stateStopsRoom(5,room)
            self.stateStopsRoom(7,room)
            
            self.roomPrimesState(room,6,0.05) #bind
            self.roomPreventsState(room,8) 


    def stateStopsShapeFact(self,state,fact):
        synWeight = -20.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = state*5+fromOff
                        toNeuron = fact*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.shapeFactCells, 
                            inhConnector,'inhibitory')

    def factPreventsState(self,fact,state):
        synWeight = -10.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = fact*5+fromOff
                        toNeuron = state*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.shapeFactCells,self.finiteSACells, 
                            inhConnector,'inhibitory')

    def factPrimesState(self,fact,state):
        synWeight = 0.05
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = fact*5+fromOff
                        toNeuron = state*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.shapeFactCells,self.finiteSACells, 
                            excConnector,'excitatory')

    def connectFSAAndShapeFacts(self):
        for fact in range (0,4):
            self.stateStopsShapeFact(1,fact)
            self.stateStopsShapeFact(2,fact)
            self.factPreventsState(fact,2)
            self.factPrimesState(fact,4) #foundShape
            self.stateStopsShapeFact(5,fact)

            self.factPrimesState(fact,6) #resetBindNets
            self.stateStopsShapeFact(7,fact) #turnOffBindNets
            self.factPreventsState(fact,8) 

    def connectShapeToBefore(self):
        #each shapeFact primes its twin in beforeShape
        synWeight = 0.25
        connector = []
        for shape in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                        fromNeuron = shape*5+fromOff
                        toNeuron = shape*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.shapeFactCells,self.beforeShapeCells, 
                            excConnector,'excitatory')

    def statePrimesBefore(self, state, shape):
        synWeight = 0.15
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = state*5+fromOff
                toNeuron = shape*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.beforeShapeCells, 
                            excConnector,'excitatory')

    def statePreventsBefore(self, state,beforeShape):
        synWeight = -10.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = state*5+fromOff
                        toNeuron = beforeShape*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.beforeShapeCells, 
                            inhConnector,'inhibitory')

    def beforePrimesState(self, beforeShape, state):
        synWeight = 0.15
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = beforeShape*5+fromOff
                toNeuron = state*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.beforeShapeCells,self.finiteSACells,
                            excConnector,'excitatory')

    def connectFSAAndBefore(self):
        for shape in range (0,4):
            self.statePrimesBefore(4,shape)
            self.statePreventsBefore(0,shape)
            self.statePreventsBefore(1,shape)
            self.beforePrimesState(shape,5)


    def setNamedShapeOn(self,shapeNumber):
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron+shapeNumber*5,synWeight,DELAY)]
        excConnector = FromListConnector(connector) 
        self.nealProjection(self.namedInput,self.namedShapeCells,
                            excConnector,'excitatory') 

    def setTestInputGetPrior(self):
        self.setTestInputBind1()
        self.startFSA()
        self.setNamedShapeOn(0)

    #---Functions for explore
    def allocateViewedFactNeurons(self):
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}

        self.viewedFactCells=Population(20,IF_curr_exp,cp)

    def allocateExploreRoomNeurons(self):
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}

        self.exploreRoomCells=Population(55,IF_curr_exp,cp)
        #room 0-3,beforeExplore, exploring, doneExplore, then room 0-3

    def allocateTestNeurons(self):
        cp = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       'i_offset' : 0.0}

        self.testFactCells=Population(20,IF_curr_exp,cp)

	cell_params = {'tau_refrac' : 5.0, 'v_rest' : -65.0,
                    'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
                    'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                    'i_offset' : 0.0, 'cm': 0.1}
        self.testGoalCells=Population(5,IF_curr_exp,cell_params)

    def makeViewedFactCAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.viewedFactCells,self.viewedFactCells, 
                            excConnector,'excitatory')

    def makeExploreRoomCAs(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,11):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.exploreRoomCells, 
                            excConnector,'excitatory')

    def expRoomStopsExpRoom(self,fromRoom,toRoom):
        synWeight = -10.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = fromRoom*5+fromOff
                toNeuron = toRoom*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.exploreRoomCells, 
                            inhConnector,'inhibitory')


    def expRoomPrimesExpRoom(self, fromRoom, toRoom):
        synWeight = 0.3
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = fromRoom*5+fromOff
                toNeuron = toRoom*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.exploreRoomCells,
                            excConnector,'excitatory')

    def expRoomStartsExpRoom(self, fromRoom, toRoom):
        synWeight = 1.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = fromRoom*5+fromOff
                toNeuron = toRoom*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.exploreRoomCells,
                            excConnector,'excitatory')

    def connectExploreRooms(self):
        self.expRoomPrimesExpRoom(4,0)
        self.expRoomPrimesExpRoom(1,2)
        self.expRoomPrimesExpRoom(2,3)
        self.expRoomStopsExpRoom(0,4)
        self.expRoomStopsExpRoom(1,0)
        self.expRoomStopsExpRoom(2,1)
        self.expRoomStopsExpRoom(3,2)

        #automata in middle of exploreRoom
        self.expRoomPrimesExpRoom(3,6)
        self.expRoomPrimesExpRoom(4,5)  #4 beforeExplore -> exploring
        self.expRoomStopsExpRoom(6,3)
        self.expRoomStopsExpRoom(3,6)   #while third room on not done
        self.expRoomStartsExpRoom(10,6) #lastNext makes done
        #self.expRoomStartsExpRoom(9,6) #undone test stop at room2

        self.expRoomStopsExpRoom(6,5)   #doneExplore stops exploring
        self.expRoomStopsExpRoom(6,10)   #done stops last next  
        self.expRoomStopsExpRoom(6,9)   #prevents next  
        #self.expRoomStopsExpRoom(6,8)   #prevents next  
        #self.expRoomStopsExpRoom(6,7)   #prevents next  


        #Next rooms
        self.expRoomStartsExpRoom(0,7)
        self.expRoomStartsExpRoom(7,1)
        self.expRoomStopsExpRoom(0,1)  #0 prevents 1

        self.expRoomStopsExpRoom(8,7)
        self.expRoomStartsExpRoom(1,8)
        self.expRoomStartsExpRoom(8,2)
        self.expRoomStopsExpRoom(1,2)

        self.expRoomStopsExpRoom(9,8)
        self.expRoomStartsExpRoom(2,9)
        self.expRoomStartsExpRoom(9,3)
        self.expRoomStopsExpRoom(2,3)

        self.expRoomStartsExpRoom(3,10)
        self.expRoomStopsExpRoom(10,9)


    def connectExploreRoomToBefore(self):
        #beforeExplore and exploring prevents befores
        synWeight = -10.0
        connector = []
        for fact in range (0,4):
            for fromOff in range (20,30):
                for toOff in range (0,5):
                    fromNeuron = fromOff
                    toNeuron = fact*5+toOff
                    connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.beforeShapeCells, 
                            inhConnector,'inhibitory')

    def connectShapeFactToViewedFacts(self):
        #synWeight = 8.0
        synWeight = 1.0
        connector = []
        for fact in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                        fromNeuron = fromOff + fact*5
                        toNeuron = toOff + fact*5
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.shapeFactCells, self.viewedFactCells,
                            excConnector,'excitatory')

    def connectExploreRoomToShapeFact(self):
        #synWeight = -10.0
        synWeight = -10.0
        connector = []
        for fromOff in range (20,25):
            for fact in range (0,4):
                for toOff in range (0,5):
                        fromNeuron = fromOff
                        toNeuron = fact*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        for fromOff in range (30,35):
            for fact in range (0,4):
                for toOff in range (0,5):
                        fromNeuron = fromOff
                        toNeuron = fact*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.externalShapeFactCells, 
                            inhConnector,'inhibitory')


    def connectExploreRoomToRoom(self):
        synWeight = 8.0
        connector = []
        for cCA in range (0,4): 
            for fromOff in range (0,5):
                for toOff in range (0,5):
                        fromNeuron = cCA*5+fromOff
                        toNeuron = cCA*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.roomCells, 
                            excConnector,'excitatory')

    def shapeFactPrimesExploreRoom(self, fact, room):
        print "used"
        synWeight = 0.5
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = fact*5+fromOff
                toNeuron = room*5+toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.shapeFactCells,self.exploreRoomCells,
                            excConnector,'excitatory')

    def stateStopsExpRoom(self,state,room):
        synWeight = -20.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                        fromNeuron = state*5+fromOff
                        toNeuron = room*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.finiteSACells,self.exploreRoomCells, 
                            inhConnector,'inhibitory')

    def connectFSAAndExploreRoom(self):
        for room in range (0,4):
            self.stateStopsExpRoom(7,room)

        #fsa in explore room exploring (5) prevents fsa 4
        synWeight = -20.0
        connector = []
        for fromOff in range (25,30):
            for toOff in range (20,25):
                fromNeuron = fromOff
                toNeuron = toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells, self.finiteSACells,
                            inhConnector,'inhibitory')

    def expRoomStopsViewedFact(self,fact):
        synWeight = -20.0
        connector = []
        for fromOff in range (30,35):
            for toOff in range (0,5):
                        fromNeuron = fromOff
                        toNeuron = fact*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.viewedFactCells, 
                            inhConnector,'inhibitory')

    def exploreRoomToViewedFacts(self):
        for fact in range (0,4):
            self.expRoomStopsViewedFact(fact) #doneExploring

    def connectExploreRoomToInBefore(self):
        #before explore ane exploring prevent inBeforeRoom
        synWeight = -20.0
        connector = []
        for fact in range (0,4):
            for fromOff in range (20,30):
                for toOff in range (0,5):
                    fromNeuron = fromOff
                    toNeuron = fact*5+toOff
                    connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]

        inhConnector = FromListConnector(connector)
        self.nealProjection(self.exploreRoomCells,self.inBeforeRoomCells, 
                            inhConnector,'inhibitory')

    def beforeShapePreventsInBefore(self,fromShape,toShape):
        synWeight = -20.0
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = fromOff+(fromShape*5)
                toNeuron = toOff+(toShape*5)
                connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]

        inhConnector = FromListConnector(connector)
        self.nealProjection(self.beforeShapeCells, self.inBeforeRoomCells, 
                            inhConnector,'inhibitory')

    def connectBeforeShapeToInBefore(self):
        self.beforeShapePreventsInBefore(0,1)
        self.beforeShapePreventsInBefore(0,2)
        self.beforeShapePreventsInBefore(0,3)

        self.beforeShapePreventsInBefore(1,0)
        self.beforeShapePreventsInBefore(1,2)
        self.beforeShapePreventsInBefore(1,3)

        self.beforeShapePreventsInBefore(2,0)
        self.beforeShapePreventsInBefore(2,1)
        self.beforeShapePreventsInBefore(2,3)

        self.beforeShapePreventsInBefore(3,0)
        self.beforeShapePreventsInBefore(3,1)
        self.beforeShapePreventsInBefore(3,2)


    def startExploreFSA(self,externalSource):
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron+20,synWeight,DELAY)]
            #connector=connector+[(0,toNeuron+25,synWeight,DELAY)] #doneExplore
        excConnector = FromListConnector(connector) 
        self.nealProjection(externalSource,self.exploreRoomCells,
                            excConnector,'excitatory') 

    #---- stand alone functions
    def connectVisionFactsToShapeFacts(self):
        synWeight = 12.0
        connector = []
        for fact in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                        fromNeuron = fromOff + fact*5
                        toNeuron = toOff + fact*5
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.testFactCells,self.shapeFactCells, 
                            excConnector,'excitatory')

    #undone move to cabot3
    def connectViewedFactsToVisionFacts(self):
        synWeight = -10.0
        connector = []
        for fact in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                        fromNeuron = fact*5+fromOff
                        toNeuron = fact*5+toOff
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        inhConnector = FromListConnector(connector)
        self.nealProjection(self.viewedFactCells,self.testFactCells, 
                            inhConnector,'inhibitory')

    #undone move to cabot3
    def connectGoalToExploreRoom(self):
        synWeight = 0.2
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                fromNeuron = fromOff 
                toNeuron = toOff
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
                toNeuron = toOff+25
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]

        excConnector = FromListConnector(connector)
        self.nealProjection(self.testGoalCells,self.exploreRoomCells, 
                            excConnector,'excitatory')

    def setCogMapExploreInputs(self):
        #spikeTimes = [[i for i in range(7,20,5)],] 
        time = 50
        spikes = [[time]]
        self.startExploreInput=Population(1,SpikeSourceArray,
                                          {'spike_times':spikes})

        time = 120
        spikes = [[time]]
        self.setGoalInput=Population(1,SpikeSourceArray,{'spike_times':spikes})

        time = 300
        spikes = [[time]]
        self.vis1Input=Population(1,SpikeSourceArray,{'spike_times':spikes})

        time = 800
        spikes = [[time]]
        self.vis2Input=Population(1,SpikeSourceArray,{'spike_times':spikes})

        time = 1000
        spikes = [[time]]
        self.vis3Input=Population(1,SpikeSourceArray,{'spike_times':spikes})

        time = 1200
        spikes = [[time]]
        self.vis4Input=Population(1,SpikeSourceArray,{'spike_times':spikes})

    def setCogMapFullInputs(self):
        self.setCogMapExploreInputs()
        time = 1600
        spikes = [[time]]
        self.namedInput=Population(1,SpikeSourceArray,{'spike_times':spikes})
        self.startFSAInput=Population(1,SpikeSourceArray,{'spike_times':spikes})


    def setTestCAs(self):
        synWeight = 2.5
        connector = []
        for fromOff in range (0,5):
            for toOff in range (0,5):
                if (fromOff != toOff):
                    fromNeuron = fromOff
                    toNeuron = toOff
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.testGoalCells,self.testGoalCells, 
                            excConnector,'excitatory')

        synWeight = 8.0
        connector = []
        for fact in range (0,4):
            for fromOff in range (0,5):
                for toOff in range (0,5):
                    if (fromOff != toOff):
                        fromNeuron = fromOff + fact*5
                        toNeuron = toOff + fact*5
                        connector=connector+[(fromNeuron,toNeuron,synWeight,
                                              DELAY)]
        excConnector = FromListConnector(connector)
        self.nealProjection(self.testFactCells,self.testFactCells, 
                            excConnector,'excitatory')

    def connectInputToTestGoal(self):
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron,synWeight,DELAY)]
        excConnector = FromListConnector(connector) 
        self.nealProjection(self.setGoalInput,self.testGoalCells,
                            excConnector,'excitatory') 

    def connectInputToTestExplore(self):
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron+20,synWeight,DELAY)]
            #connector=connector+[(0,toNeuron+25,synWeight,DELAY)] #doneExplore
        excConnector = FromListConnector(connector) 
        self.nealProjection(self.startExploreInput,self.exploreRoomCells,
                            excConnector,'excitatory') 

    def connectInputToTestFact(self,input,fact):
        synWeight = 20.0
        connector = []
        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron+fact*5,synWeight,DELAY)]
        excConnector = FromListConnector(connector) 
        self.nealProjection(input,self.testFactCells,
                            excConnector,'excitatory') 

    def setTestInputExplore(self):
        self.connectInputToTestGoal()
        self.connectInputToTestExplore()
        self.connectInputToTestFact(self.vis1Input,1)
        self.connectInputToTestFact(self.vis2Input,0)
        self.connectInputToTestFact(self.vis3Input,2)
        self.connectInputToTestFact(self.vis4Input,3)
        return

    def setTestInputFull(self):
        self.setTestInputExplore()
        self.startFSA()
        self.setNamedShapeOn(2)

    #---Functions for all components
    def allocateCogMapNeurons(self):
        self.allocateRoomNeurons()
        self.allocateShapeFactNeurons()
        self.allocateExternalShapeFactNeurons()
        self.allocateNamedShapeNeurons()
        self.allocateFSANeurons()
        self.allocateBeforeShapeNeurons()
        if (self.fStandAlone):
            self.allocateTestNeurons()
        self.allocateViewedFactNeurons()
        self.allocateExploreRoomNeurons()
        self.allocateInBeforeRoomNeurons()
 
    def setCogMapConnections(self):
        #binding
        self.makeRoomCAs()
        self.makeShapeFactCAs()
        self.connectRoomsAndShapeFacts()
        self.connectExternalFactsToShapeFacts()
       
        #get prior room shape
        self.makeNamedCAs()
        self.makeFSACAs()
        self.makeBeforeCAs()
        self.connectNamedToShapeFacts()
        self.connectFSA()
        self.finiteSATurnsOffNamed()
        self.roomsPrimeEarlierRooms()
        self.roomsStopLaterRooms()
        self.connectRoomsAndFSA()
        self.connectFSAAndShapeFacts()
        self.connectShapeToBefore()
        self.connectFSAAndBefore()
        #self.setTestInputGetPrior()
        
        #explore 
        self.makeViewedFactCAs()
        self.makeExploreRoomCAs()
        self.connectExploreRooms()
        if (self.fStandAlone):
            self.setTestCAs()
            self.connectVisionFactsToShapeFacts() 
        #self.setTestInputExplore()
        self.connectExploreRoomToBefore()
        #undoneself.connectGoalToExploreRoom()#move to cabot3
        #undoneself.connectViewedFactsToVisionFacts() #move to cabot3
        self.connectShapeFactToViewedFacts()
        self.connectExploreRoomToShapeFact()
        self.connectExploreRoomToRoom()
        #use connectRoomsAndFSA from before
        #use connectFSAAndShapeFacts from before
        self.connectFSAAndExploreRoom()
        self.exploreRoomToViewedFacts()
        self.connectExploreRoomToInBefore()
        self.connectBeforeShapeToInBefore()

        if (self.fStandAlone):
            self.setTestInputFull()


    def setCogMapRecording(self):
        if self.simulator_name == 'spiNNaker':
            self.roomCells.record()
            self.shapeFactCells.record()
            self.externalShapeFactCells.record()
            self.namedShapeCells.record()
            self.finiteSACells.record()
            self.beforeShapeCells.record()
            self.inBeforeRoomCells.record()
            if (self.fStandAlone):
                self.testFactCells.record()
                self.testGoalCells.record()
            self.viewedFactCells.record()
            self.exploreRoomCells.record()

        elif self.simulator_name == 'nest':
            self.roomCells.record('spikes')

    def printCogMapResults(self):
        #self.roomCells.printSpikes("results/cmRoom.sp")
        self.shapeFactCells.printSpikes("results/cmShapeFacts.sp")
        self.externalShapeFactCells.printSpikes("results/cmExternalShapeFacts.sp")
        self.namedShapeCells.printSpikes("results/cmNamedShape.sp")
        self.finiteSACells.printSpikes("results/cmFSA.sp")
        self.beforeShapeCells.printSpikes("results/cmBeforeShape.sp")
        self.inBeforeRoomCells.printSpikes("results/cmInBeforeRoom.sp")
        if (self.fStandAlone):
            self.testFactCells.printSpikes("results/testFact.sp")
            self.testGoalCells.printSpikes("results/testGoal.sp")
        #self.viewedFactCells.printSpikes("results/cmViewedFact.sp")
        #self.exploreRoomCells.printSpikes("results/cmExploreRoom.sp")


    #----Access To Cells for external subnets
    def getExploreRoomCells(self):
        return self.exploreRoomCells

    def getShapeFactCells(self):
        return self.externalShapeFactCells

    def getViewedFactCells(self):
        return self.viewedFactCells

    def getFSACells(self):
        return self.finiteSACells

    def getNamedShapeCells(self):
        return self.namedShapeCells

    def getInBeforeRoomCells(self):
        return self.inBeforeRoomCells

    def getBeforeShapeCells(self):
        return self.beforeShapeCells
