"""
The vision class without the connections.
"""

#imports for PyNN
from pyNN.utility import get_script_args#, normalized_filename
#from neo.io import PyNNTextIO
#from pyNN.nest import *
from pyNN.spiNNaker import *

#import for spike injection
import spynnaker_external_devices_plugin.pyNN as externaldevices
from spinnman.messages.eieio.eieio_prefix_type import EIEIOPrefixType

from nealParams import *
from retinaConnections import get3x3OnExcite, get3x3OnInhib
from retinaConnections import get3x3OffExcite,get3x3OffInhib
from retinaConnections import get6x6OnExcite, get6x6OnInhib
from retinaConnections import get6x6OffExcite, get6x6OffInhib
from retinaConnections import get9x9OnExcite, get9x9OnInhib
from retinaConnections import get9x9OffExcite,get9x9OffInhib

from v1Connections import getHEdgeTopOrBottomFrom3x3OffExcite
from v1Connections import getHEdgeBottomFrom6x6OffInhib
from v1Connections import getHEdgeBottomFrom9x9OffInhib
from v1Connections import getHEdgeTopFrom6x6OffInhib
from v1Connections import getHEdgeTopFrom9x9OffInhib
from v1Connections import getVEdgeLeftOrRightFrom3x3OffExcite
from v1Connections import getVEdgeLeftFrom9x9OffInhib
from v1Connections import getVEdgeRightFrom9x9OffInhib
from v1Connections import getSEdgeTopOrBottomFrom3x3OffExcite
from v1Connections import getSEdgeTopOrBottomFrom6x6OffInhib
from v1Connections import getSEdgeTopOrBottomFrom9x9OffInhib
from v1Connections import getSEdgeTopFrom6x6OffInhib
from v1Connections import getSEdgeTopFrom9x9OffInhib
from v1Connections import getSEdgeBottomFrom9x9OffInhib


class visionArea:

    def __init__(self, simName):
        self.simulator_name = simName
        self.threeOn = False
        self.threeOff = True
        self.sixOn = False
        self.sixOff = False
        self.nineOn = False
        self.nineOff = True

        self.hEdgeTop = True
        self.hEdgeBottom = True
        self.vEdgeLeft = False
        self.vEdgeRight = False
        self.sEdgeTop = False
        self.sEdgeBottom = False
        self.andAngle = False
        self.orAngle = False


    def setWhatsOn(self,theSwitch):
        if (theSwitch == 1): #none
                self.threeOn = False
                self.threeOff = False
                self.sixOn = False
                self.sixOff = False
                self.nineOn = False
                self.nineOff = False

                self.hEdgeTop = False
                self.hEdgeBottom = False
                self.vEdgeLeft = False
                self.vEdgeRight = False
                self.sEdgeTop = False
                self.sEdgeBottom = False
                
        elif (theSwitch == 2): #just threeOff, nineOff, and hedges 
                self.threeOn = False
                self.threeOff = True
                self.sixOn = False
                self.sixOff = False
                self.nineOn = False
                self.nineOff = True

                self.hEdgeTop = True
                self.hEdgeBottom = True
                self.vEdgeLeft = False
                self.vEdgeRight = False
                self.sEdgeTop = False
                self.sEdgeBottom = False
                self.andAngle = False
                self.orAngle = False

        elif (theSwitch == 3): #just, 3Off, 6off, 9Off, hedges, sedges
                self.threeOn = False
                self.threeOff = True
                self.sixOn = False
                self.sixOff = True
                self.nineOn = False
                self.nineOff = True

                self.hEdgeTop = True
                self.hEdgeBottom = True
                self.vEdgeLeft = False
                self.vEdgeRight = False
                self.sEdgeTop = True
                self.sEdgeBottom = True
                self.andAngle = False
                self.orAngle = False

        elif (theSwitch == 4): #just, 3Off, 6off, 9Off, hedges, sedges, vedge
                self.threeOn = True
                self.threeOff = True
                self.sixOn = False
                self.sixOff = True
                self.nineOn = False
                self.nineOff = True

                self.hEdgeTop = True
                self.hEdgeBottom = True
                self.vEdgeLeft = True
                self.vEdgeRight = True
                self.sEdgeTop = True
                self.sEdgeBottom = True
                self.andAngle = False
                self.orAngle = True

        else:
                self.threeOn = True
                self.threeOff = True
                self.sixOn = True
                self.sixOff = True
                self.nineOn = True
                self.nineOff = True

                self.hEdgeTop = True
                self.hEdgeBottom = True
                self.vEdgeLeft = True
                self.vEdgeRight = True
                self.sEdgeTop = True
                self.sEdgeBottom = True
                self.andAngle = True
                self.orAngle = True

    def nealProjection(self,preNeurons,postNeurons, connectorList,inhExc):
        if self.simulator_name == 'spiNNaker':
            Projection(preNeurons, postNeurons, connectorList,target=inhExc)
        elif self.simulator_name == 'nest':
            Projection(preNeurons, postNeurons, connectorList)
        else:
            print 'bad simulator for nealProjection'

    #------print functions
    def printPopulationFirings(self,receptorPop):
        totalNeurons = neuronsPerSubnet
        for neuronNum in range (0,totalNeurons-1):
            if (neuronNum < totalNeurons):
                outAssembly = Assembly(receptorPop[neuronNum,(neuronNum+1)])
            else: 
                outAssembly = Assembly(receptorPop[neuronNum,0])
            outDat = outAssembly.get_data()
            seg = outDat.segments[0]
            spikeTrains = seg.spiketrains
            if len(spikeTrains) > 0:
                times = spikeTrains[0].times
                if len(times) > 0:
                    print neuronNum, len(times),
                    for time in times:
                        base = time.base
                        print base,
                    print " "

    def printReceptors(self):
        if self.threeOn:
            print "3x3 On"
            self.printPopulationFirings(self.threeOnCells)
        if self.threeOff:
            print "3x3 Off"
            self.printPopulationFirings(self.threeOffCells)
        if self.sixOn:
            print "6x6 On"
            self.printPopulationFirings(self.sixOnCells)
        if self.sixOff:
            print "6x6 Off"
            self.printPopulationFirings(self.sixOffCells)
        if self.nineOn:
            print "9x9 On"
            self.printPopulationFirings(self.nineOnCells)
        if self.nineOff:
            print "9x9 Off"
            self.printPopulationFirings(self.nineOffCells)

    def printV1(self):
        if self.hEdgeTop:
            print "HEdgeBottom"
            self.printPopulationFirings(self.hEdgeBottomCells)
        if self.hEdgeTop:
            print "HEdgeTop"
            self.printPopulationFirings(self.hEdgeTopCells)


    def printFact(self,factCells,factName):
        if self.simulator_name == 'spiNNaker':
            factCells.printSpikes('results/' + factName)

        elif self.simulator_name == 'nest':
            for neuronNum in range (0,4):
                outAssembly = Assembly(factCells[neuronNum,(neuronNum+1)])
                outDat = outAssembly.get_data()
                seg = outDat.segments[0]
                spikeTrains = seg.spiketrains
                if len(spikeTrains) > 0:
                    times = spikeTrains[0].times
                    if len(times) > 0:
                        print neuronNum, len(times),
                        for time in times:
                            base = time.base
                            print base,
                        print " "
        
    def printObjects(self):
        print 'Pyramid Left'
        self.printFact(self.pyramidLeftCells,"pyramidLeft.sp") 
        print 'Pyramid Right'
        self.printFact(self.pyramidRightCells,"pyramidRight.sp")
        print 'Stalactite Left'
        self.printFact(self.stalactiteLeftCells,"stalactiteLeft.sp")
        print 'Stalactite Right'
        self.printFact(self.stalactiteRightCells,"stalactiteRight.sp")

    def printVisionNets(self):
        if self.simulator_name == 'spiNNaker':
            #self.threeOnCells.printSpikes('results/vis3N.sp')
            #self.threeOffCells.printSpikes('results/vis3F.sp')
            #self.sixOnCells.printSpikes('results/vis6N.sp')
            #self.sixOffCells.printSpikes('results/vis6F.sp')
            #self.nineOnCells.printSpikes('results/vis9N.sp')
            #self.nineOffCells.printSpikes('results/vis9F.sp')
            #self.hEdgeTopCells.printSpikes('results/visHT.sp')
            #self.hEdgeBottomCells.printSpikes('results/visHB.sp')
            #self.sEdgeTopCells.printSpikes('results/visST.sp')
            #self.sEdgeBottomCells.printSpikes('results/visSB.sp')
            #self.vEdgeLeftCells.printSpikes('results/visVL.sp')
            #self.vEdgeRightCells.printSpikes('results/visVR.sp')
            #self.orAngleCells.printSpikes('results/visOA.sp')
            self.visionFactCells.printSpikes('results/visFacts.sp')
        elif self.simulator_name == 'nest':
            #self.printReceptors()
            #self.printV1()
            self.printObjects()
        elif self.simulator_name == 'out':
            print '3On'
            outAss = Assembly(self.threeOnCells[4,1411])
            #outDat = self.threeOnCells.get_data()
            outDat = outAss.get_data()
            for seg in outDat.segments:
                print(seg.spiketrains)
            print '3Off'
            #outAss = Assembly(self.threeOffCells[4,1411])
            outDat = self.threeOffCells.get_data()
            #outDat = outAss.get_data()
            for seg in outDat.segments:
                print(seg.spiketrains)


    #---create neurons
    def allocateVisualInputNeurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
               'i_offset' : 0.1}

        self.inputCells=Population(neuronsPerSubnet, IF_curr_exp, cell_params)


    def allocateSpinnakerVisualInputNeurons(self):
        cell_params_spike_injector_with_prefix = {'host_port_number' : 12346,
                            'host_ip_address'  : "localhost",
                            'virtual_key'      : 0x90800,
                            'prefix'           : 9,
                            'prefix_type': EIEIOPrefixType.UPPER_HALF_WORD}

        self.bwVisualInputSource = Population(neuronsPerVisionSubnet,
                        externaldevices.ReverseIpTagMultiCastSource,
                        cell_params_spike_injector_with_prefix, 
                        label='bw_spike_injector')


        cell_params_spike_injector_with_prefixColour={'host_port_number': 12347,
                            'host_ip_address'  : "localhost",
                            'virtual_key'      : 0x50800,
                            'prefix'           : 5,
                            'prefix_type': EIEIOPrefixType.UPPER_HALF_WORD}
        self.colourVisualInputSource = Population(1200,
                        externaldevices.ReverseIpTagMultiCastSource,
                        cell_params_spike_injector_with_prefixColour, 
                        label='colour_spike_injector')


    def allocateRetinaNeurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
               'i_offset' : 0.1}
        if self.threeOn:
            self.threeOnCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.threeOnCells=Population(1,IF_curr_exp,cell_params)

        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
                       #'i_offset' : -0.5}
                       'i_offset' : 0.0}
        if self.threeOff:
            self.threeOffCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.threeOffCells=Population(1,IF_curr_exp,cell_params)
        if self.nineOn:
            self.nineOnCells = Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.nineOnCells = Population(1,IF_curr_exp,cell_params)
        if self.nineOff:
            self.nineOffCells= Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.nineOffCells= Population(1,IF_curr_exp,cell_params)

        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
               'i_offset' : 0.0}
        if self.sixOn:
            self.sixOnCells = Population(neuronsPerSubnet,IF_curr_exp, cell_params)
        else:
            self.sixOnCells = Population(1,IF_curr_exp, cell_params)
        if self.sixOff:
            self.sixOffCells= Population(neuronsPerSubnet,IF_curr_exp, cell_params)
        else:
            self.sixOffCells= Population(1,IF_curr_exp, cell_params)
        

    def allocateV1Neurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
               'i_offset' : 0.1}
        if self.hEdgeBottom:
            self.hEdgeBottomCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.hEdgeBottomCells=Population(1,IF_curr_exp,cell_params)
        if self.hEdgeTop:
            self.hEdgeTopCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.hEdgeTopCells=Population(1,IF_curr_exp,cell_params)
        if self.vEdgeLeft:
            self.vEdgeLeftCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.vEdgeLeftCells=Population(1,IF_curr_exp,cell_params)
        if self.vEdgeRight:
            self.vEdgeRightCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.vEdgeRightCells=Population(1,IF_curr_exp,cell_params)

        if self.sEdgeTop:
            self.sEdgeTopCells=Population(neuronsPerSubnet,IF_curr_exp,cell_params)
        else:
            self.sEdgeTopCells=Population(1,IF_curr_exp,cell_params)
        if self.sEdgeBottom:
            self.sEdgeBottomCells=Population(neuronsPerSubnet,IF_curr_exp,
                                             cell_params)
        else:
            self.sEdgeBottomCells=Population(1,IF_curr_exp,cell_params)

        if self.orAngle:
            self.orAngleCells=Population(neuronsPerSubnet,IF_curr_exp,
                                         cell_params)
        else:
            self.orAngleCells=Population(1,IF_curr_exp,cell_params)


    def allocateObjRecNeurons(self):
        cell_params = {'tau_refrac' : 3.0, 'v_rest' : -65.0,
               'v_thresh' : -51.0,  'tau_syn_E'  : 2.0, 
               'tau_syn_I': 5.0,    'v_reset'    : -70.0, 
               'i_offset' : 0.0}
               #'i_offset' : 0.1} 4chip
        self.visionFactCells = Population(90,IF_curr_exp,cell_params)
        #crh fact names
        #bluePyrLeft,bluePyrRight,redStalLeft,redStalRight, 15-19
        #bluePyrCentre, #redStalCentre,doorLeft, endDoorCentre 35-39 ,
        #doorCentre,doorRight,red,blue  55-59
        #redPyrLeft, redPyrCtr, redPyrRight, blueStalLeft 75-79
        #blueStalCtr,blueStalRight 85-89
        
    def allocateVisionNeurons(self):
        if simulator_name == 'spiNNaker':
            self.allocateSpinnakerVisualInputNeurons()
            #print "stub"
        elif simulator_name == 'nest':
            self.allocateVisualInputNeurons()
        else:
            print 'bad simulator for AllocateVisionNeurons'
        self.allocateRetinaNeurons()
        self.allocateV1Neurons()
        self.allocateObjRecNeurons()


    #----------------setup input to retina connections
    #this is needed for spinnaker with more than 2048 inputcells
    def removeLargerInputs(self,connList):
        smallConnections = []
        for connection in connList:
            if (connection[0] < 2048):
                smallConnections = smallConnections + [connection]
        return smallConnections
            
    def modifyLargerInputs(self,connList):
        largeConnections = []
        for connection in connList:
            if (connection[0] >= 2048):
                newConnection = [(connection[0]-2048,connection[1],
                                  connection[2],connection[3])]
                largeConnections = largeConnections + [newConnection]
        return largeConnections
        
    def nealVisionInputProjection(self,inputCells,retinaCells,
                                       connList,excOrIn):
        if ((self.simulator_name == 'spiNNaker') and (neuronsPerSubnet > 2048)):
            #print connList
            firstConnList = self.removeLargerInputs(connList)
            connector = FromListConnector(firstConnList)
            self.nealProjection(inputCells,retinaCells,connector,excOrIn)
            secondConnList = self.modifyLargerInputs(connList)
            connector = FromListConnector(secondConnList)
            #self.nealProjection(self.inputCells2,retinaCells,connector,excOrIn)
            print "big"
        else:
            connector = FromListConnector(connList)
            self.nealProjection(inputCells,retinaCells,connector,excOrIn)


    def makeInputToRetSynapses(self):
        #3x3On
        if self.threeOn:
            excConnectorList = get3x3OnExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.bwVisualInputSource,self.threeOnCells, 
                                excConnector,'excitatory')

            inhConnectorList = get3x3OnInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.bwVisualInputSource,self.threeOnCells, 
                                inhConnector,'inhibitory')

        #3x3Off
        if self.threeOff:
            excConnectorList = get3x3OffExcite()
            self.nealVisionInputProjection(self.bwVisualInputSource, 
                                           self.threeOffCells,
                                           excConnectorList,'excitatory')

            inhConnectorList = get3x3OffInhib()
            self.nealVisionInputProjection(self.bwVisualInputSource, 
                            self.threeOffCells,inhConnectorList,'inhibitory')

        #6x6On
        if self.sixOn:
            excConnectorList = get6x6OnExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.bwVisualInputSource, self.sixOnCells, 
                                excConnector,'excitatory')
            inhConnectorList = get6x6OnInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.bwVisualInputSource, self.sixOnCells, 
                                inhConnector,'inhibitory')

        #6x6Off
        if self.sixOff:
            excConnectorList = get6x6OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.bwVisualInputSource, self.sixOffCells, 
                                excConnector,'excitatory')
            inhConnectorList = get6x6OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.bwVisualInputSource, self.sixOffCells, 
                                inhConnector,'inhibitory')

        #9x9On
        if self.nineOn:
            excConnectorList = get9x9OnExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.bwVisualInputSource, self.nineOnCells,
                                excConnector,'excitatory')
            inhConnectorList = get9x9OnInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.bwVisualInputSource, self.nineOnCells, 
                                inhConnector,'inhibitory')

        #9x9off
        if self.nineOff:
            excConnectorList = get9x9OffExcite()
            self.nealVisionInputProjection(self.bwVisualInputSource, 
                                           self.nineOffCells, 
                                           excConnectorList,'excitatory')
            inhConnectorList = get9x9OffInhib()
            self.nealVisionInputProjection(self.bwVisualInputSource, 
                                           self.nineOffCells, 
                                           inhConnectorList,'inhibitory')


    def colourPreventsObjectFact(self,colourStart,leftStart,rightEnd,factNum):
        synWeight = -10.0
        connector = []
        for cRow in range (0,20):
            for cCol in range (int(20*leftStart),int(20*rightEnd)):
                for toOffset in range (0,5):
                    fromNeuron = cRow*20+ cCol+colourStart
                    toNeuron = toOffset + (factNum*5)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        inhConnectorList = FromListConnector(connector) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            inhConnectorList,'inhibitory')

    def makeColourToObjFactSynapses(self):
        #green stops left bluePyr, redStal
        self.colourPreventsObjectFact(400,0.0,0.4,0)
        self.colourPreventsObjectFact(400,0.0,0.4,2)

        #green stops ctr bluePyr, redStal, redPyr, blueStal
        self.colourPreventsObjectFact(400,0.3,0.7,4)
        self.colourPreventsObjectFact(400,0.3,0.7,5)
        self.colourPreventsObjectFact(400,0.3,0.7,13)
        self.colourPreventsObjectFact(400,0.3,0.8,16)

        #green stops rt bluePyr, redStal ,rePyr
        self.colourPreventsObjectFact(400,0.7,1.0,1)
        self.colourPreventsObjectFact(400,0.7,1.0,3)
        self.colourPreventsObjectFact(400,0.7,1.0,14)


        #red stops left bluePyr and blueStal
        self.colourPreventsObjectFact(0,0.0,0.4,0)
        self.colourPreventsObjectFact(0,0.0,0.4,15)

        #red stops centre bluePyr, blueStal
        self.colourPreventsObjectFact(0,0.3,0.7,4)
        self.colourPreventsObjectFact(0,0.3,0.7,16)

        #blue stops left redStal, redPyr
        self.colourPreventsObjectFact(800,0.0,0.4,2)
        self.colourPreventsObjectFact(800,0.0,0.4,12)

        #blue stops centre redStal, redPyr
        self.colourPreventsObjectFact(800,0.3,0.7,5)
        self.colourPreventsObjectFact(800,0.3,0.7,13)

        #Have the colour prime appropriate objects
        objWt = 0.05
        #objWt = 0.06
        #blue to left pyramid
        bPyrLeftConnectors=self.connectV1FeatureToObject2(objWt,0,0.3,0,20,800)
        bPyrLeftConnectorList = FromListConnector(bPyrLeftConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            bPyrLeftConnectorList,'excitatory')

        #blue to blue center pyr
        bPyrCtrConnectors=self.connectV1FeatureToObject2(objWt,0.3,0.7,4,20,800)
        bPyrCtrConnectorList = FromListConnector(bPyrCtrConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            bPyrCtrConnectorList,'excitatory')

        #blue to right pyramid
        bPyrRightConnectors=self.connectV1FeatureToObject2(objWt,0.7,1.0,1,20,800)
        bPyrRightConnectorList = FromListConnector(bPyrRightConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            bPyrRightConnectorList,'excitatory')

        #blue to blue left stal
        bStalLeftConnectors=self.connectV1FeatureToObject2(objWt,0,0.3,15,20,800)
        bStalLeftConnectorList = FromListConnector(bStalLeftConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            bStalLeftConnectorList,'excitatory')

        #blue to blue center stal
        bStalCtrConnectors = self.connectV1FeatureToObject2(objWt,0.3,0.7,16,20,800)
        bStalCtrConnectorList = FromListConnector(bStalCtrConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            bStalCtrConnectorList,'excitatory')

        #blue to right stal
        bStalRightConnectors=self.connectV1FeatureToObject2(objWt,0.7,1.0,17,20,800)
        bStalRightConnectorList = FromListConnector(bStalRightConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            bStalRightConnectorList,'excitatory')

        #red to red left pyramid
        rPyrLeftConnectors = self.connectV1FeatureToObject2(objWt,0,0.3,12,20,0)
        rPyrLeftConnectorList = FromListConnector(rPyrLeftConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            rPyrLeftConnectorList,'excitatory')

        #red to centreRedPyr
        ctrRedPyrConnectors=self.connectV1FeatureToObject2(objWt,0.3,0.7,13,20,0)
        ctrRedPyrConnectorList = FromListConnector(ctrRedPyrConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            ctrRedPyrConnectorList,'excitatory')

        #red to rightRedpyramid
        rPyrRightConnectors=self.connectV1FeatureToObject2(objWt,0.7,1.0,14,20,0)
        rPyrRightConnectorList = FromListConnector(rPyrRightConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            rPyrRightConnectorList,'excitatory')

        #red to red left stal
        rStalLeftConnectors = self.connectV1FeatureToObject2(objWt,0,0.3,2,20,0)
        rStalLeftConnectorList = FromListConnector(rStalLeftConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            rStalLeftConnectorList,'excitatory')

        #red to centreRedStal
        ctrRedStalConnectors=self.connectV1FeatureToObject2(objWt,0.3,0.7,5,20,0)
        ctrRedStalConnectorList = FromListConnector(ctrRedStalConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            ctrRedStalConnectorList,'excitatory')

        #green to endDoorCentre
        endDoorConnectors=self.connectV1FeatureToObject2(0.05,0,0.3,7,20,400)
        endDoorConnectorList = FromListConnector(endDoorConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            endDoorConnectorList,'excitatory')
        endDoorConnectors=self.connectV1FeatureToObject2(0.05,0.7,1.0,7,20,400)
        endDoorConnectorList = FromListConnector(endDoorConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            endDoorConnectorList,'excitatory')
        endDoorConnectors=self.connectV1FeatureToObject2(-0.02,0.3,0.7,7,20,400)
        endDoorConnectorList = FromListConnector(endDoorConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            endDoorConnectorList,'inhibitory')

        #green to leftdoor
        leftDoorConnectors=self.connectV1FeatureToObject2(0.5,0,0.3,6,20,400)
        leftDoorConnectorList = FromListConnector(leftDoorConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            leftDoorConnectorList,'excitatory')

        #green to centredoor
        ctrDoorConnectors=self.connectV1FeatureToObject2(0.5,0.3,0.7,8,20,400)
        ctrDoorConnectorList = FromListConnector(ctrDoorConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            ctrDoorConnectorList,'excitatory')

        #green to rightdoor
        rtDoorConnectors=self.connectV1FeatureToObject2(0.5,0.7,1.0,9,20,400)
        rtDoorConnectorList = FromListConnector(rtDoorConnectors) 
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            rtDoorConnectorList,'excitatory')

    def makeColourToFactSynapses(self):
        weight = 10.0
        redConnectors = []
        blueConnectors = []
        for inpNeuron in range (0,400):
            for outOffset in range (0,5):
                toRedNeuron = 50+outOffset
                fromBlueNeuron = inpNeuron + 800
                toBlueNeuron = 55+outOffset
                redConnectors = redConnectors + [(inpNeuron,toRedNeuron,weight,
                                                  DELAY)]
                blueConnectors = blueConnectors + [(fromBlueNeuron,toBlueNeuron,
                                                    weight,DELAY)]

        redConnector = FromListConnector(redConnectors)
        blueConnector = FromListConnector(blueConnectors)
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            redConnector,'excitatory')
        self.nealProjection(self.colourVisualInputSource,self.visionFactCells,
                            blueConnector,'excitatory')

        self.makeColourToObjFactSynapses()




    #-------------------setup retina to V1 connections---
    def makeRetToV1Synapses(self):
        if self.hEdgeBottom:
            excConnectorList = getHEdgeTopOrBottomFrom3x3OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.threeOffCells,self.hEdgeBottomCells, excConnector,'excitatory')
            inhConnectorList = getHEdgeBottomFrom6x6OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.sixOffCells, self.hEdgeBottomCells, 
                                inhConnector,'inhibitory')
            inhConnectorList = getHEdgeBottomFrom9x9OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.nineOffCells, self.hEdgeBottomCells, 
                                inhConnector,'inhibitory')

        if self.hEdgeTop:
            excConnectorList = getHEdgeTopOrBottomFrom3x3OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.threeOffCells,self.hEdgeTopCells, 
                                excConnector,'excitatory')

            inhConnectorList = getHEdgeTopFrom6x6OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.sixOffCells, self.hEdgeTopCells, 
                                inhConnector,'inhibitory')

            inhConnectorList = getHEdgeTopFrom9x9OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.nineOffCells, self.hEdgeTopCells, 
                                inhConnector,'inhibitory')

        if self.vEdgeLeft:
            excConnectorList = getVEdgeLeftOrRightFrom3x3OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.threeOffCells,self.vEdgeLeftCells, excConnector,'excitatory')
            inhConnectorList = getVEdgeLeftFrom9x9OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.nineOffCells, self.vEdgeLeftCells, inhConnector,'inhibitory')


        if self.vEdgeRight:
            excConnectorList = getVEdgeLeftOrRightFrom3x3OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.threeOffCells,self.vEdgeRightCells, excConnector,'excitatory')
            inhConnectorList = getVEdgeRightFrom9x9OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.nineOffCells, self.vEdgeRightCells, inhConnector,'inhibitory')

        if self.sEdgeTop:
            excConnectorList = getSEdgeTopOrBottomFrom3x3OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.threeOffCells,self.sEdgeTopCells, 
                                excConnector,'excitatory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom6x6OffInhib()
            inhConnectorListT = getSEdgeTopFrom6x6OffInhib()
            inhConnectorList = inhConnectorListT+inhConnectorListTB
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.sixOffCells, self.sEdgeTopCells, 
                                inhConnector,'inhibitory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom9x9OffInhib()
            inhConnectorListT = getSEdgeTopFrom9x9OffInhib()
            inhConnectorList = inhConnectorListT+inhConnectorListTB
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.nineOffCells, self.sEdgeTopCells, inhConnector,'inhibitory')


        if self.sEdgeBottom:
            excConnectorList = getSEdgeTopOrBottomFrom3x3OffExcite()
            excConnector = FromListConnector(excConnectorList) 
            self.nealProjection(self.threeOffCells,self.sEdgeBottomCells, 
                                excConnector,'excitatory')

            inhConnectorList = getSEdgeTopOrBottomFrom6x6OffInhib()
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.sixOffCells, self.sEdgeBottomCells, 
                                inhConnector,'inhibitory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom9x9OffInhib()
            inhConnectorListB = getSEdgeBottomFrom9x9OffInhib()
            inhConnectorList = inhConnectorListB+inhConnectorListTB
            inhConnector = FromListConnector(inhConnectorList)
            self.nealProjection(self.nineOffCells, self.sEdgeBottomCells, inhConnector,'inhibitory')


    #-------------------setup Object Recognion and  V1 to ObjRec connections---
    def connectV1FeatureToObject (self, synWeight,leftStart, rightEnd,factNum):
        connector = []
        for cRow in range (0,INPSIZE):
            for cCol in range (int(INPSIZE*leftStart),int(INPSIZE*rightEnd)):
                for toOffset in range (0,5):
                    fromNeuron = cRow*INPSIZE+ cCol
                    toNeuron = toOffset + (factNum*5)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        return connector

    def connectV1FeatureToObject2 (self, synWeight,leftStart, rightEnd,factNum,
                                   inpSize,fromOffset):
        connector = []
        for cRow in range (0,inpSize):
            for cCol in range (int(inpSize*leftStart),int(inpSize*rightEnd)):
                for toOffset in range (0,5):
                    fromNeuron = cRow*inpSize+ cCol+fromOffset
                    toNeuron = toOffset + (factNum*5)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,DELAY)]
        return connector

    #main proc
    def makeV1ToObjRecSynapses(self):

        featWt = 2.0
        #bluePyrLeft
        bPyrLeftConnectors = self.connectV1FeatureToObject(featWt,0,0.3,0)
        bPyrLeftConnectorList = FromListConnector(bPyrLeftConnectors) 
        self.nealProjection(self.hEdgeBottomCells,self.visionFactCells,
                            bPyrLeftConnectorList,'excitatory')
        bPyrLeftConnectors = self.connectV1FeatureToObject(featWt,0,0.2,0)
        bPyrLeftConnectorList = FromListConnector(bPyrLeftConnectors) 
        self.nealProjection(self.sEdgeTopCells,self.visionFactCells,
                            bPyrLeftConnectorList,'excitatory')

        #Blue Pyramid Centre
        bPyrCentreConnectors = self.connectV1FeatureToObject(featWt,0.3,0.7,4)
        bPyrCentreConnectorList = FromListConnector(bPyrCentreConnectors) 
        self.nealProjection(self.hEdgeBottomCells,self.visionFactCells,
                            bPyrCentreConnectorList,'excitatory')
        bPyrCentreConnectors = self.connectV1FeatureToObject(featWt,0.2,0.6,4)
        bPyrCentreConnectorList = FromListConnector(bPyrCentreConnectors) 
        self.nealProjection(self.sEdgeTopCells,self.visionFactCells,
                            bPyrCentreConnectorList,'excitatory')

        #blue pyrRight
        bPyrRightConnectors = self.connectV1FeatureToObject(featWt,0.7,1.0,1)
        bPyrRightConnectorList = FromListConnector(bPyrRightConnectors) 
        self.nealProjection(self.hEdgeBottomCells,self.visionFactCells,
                            bPyrRightConnectorList,'excitatory')
        bPyrRightConnectors = self.connectV1FeatureToObject(featWt,0.6,1.0,1)
        bPyrRightConnectorList = FromListConnector(bPyrRightConnectors) 
        self.nealProjection(self.sEdgeTopCells,self.visionFactCells,
                            bPyrRightConnectorList,'excitatory')


        #red pyramid left 
        rPyrLeftConnectors = self.connectV1FeatureToObject(featWt,0,0.3,12)
        rPyrLeftConnectorList = FromListConnector(rPyrLeftConnectors) 
        self.nealProjection(self.hEdgeBottomCells,self.visionFactCells,
                            rPyrLeftConnectorList,'excitatory')
        rPyrLeftConnectors = self.connectV1FeatureToObject(featWt,0,0.2,12)
        rPyrLeftConnectorList = FromListConnector(rPyrLeftConnectors) 
        self.nealProjection(self.sEdgeTopCells,self.visionFactCells,
                            rPyrLeftConnectorList,'excitatory')

        #Red Pyramid Centre
        rPyrCentreConnectors = self.connectV1FeatureToObject(featWt,0.3,0.7,13)
        rPyrCentreConnectorList = FromListConnector(rPyrCentreConnectors) 
        self.nealProjection(self.hEdgeBottomCells,self.visionFactCells,
                            rPyrCentreConnectorList,'excitatory')
        rPyrCentreConnectors = self.connectV1FeatureToObject(featWt,0.2,0.6,13)
        rPyrCentreConnectorList = FromListConnector(rPyrCentreConnectors) 
        self.nealProjection(self.sEdgeTopCells,self.visionFactCells,
                            rPyrCentreConnectorList,'excitatory')

        #red pyrRight
        rPyrRightConnectors = self.connectV1FeatureToObject(featWt,0.7,1.0,14)
        rPyrRightConnectorList = FromListConnector(rPyrRightConnectors) 
        self.nealProjection(self.hEdgeBottomCells,self.visionFactCells,
                            rPyrRightConnectorList,'excitatory')
        rPyrRightConnectors = self.connectV1FeatureToObject(featWt,0.6,1.0,14)
        rPyrRightConnectorList = FromListConnector(rPyrRightConnectors) 
        self.nealProjection(self.sEdgeTopCells,self.visionFactCells,
                            rPyrRightConnectorList,'excitatory')

        #undone need blue left and redRightStal
        #blue Stalactite Centre
        bStalCentreConnectors = self.connectV1FeatureToObject(featWt,0.3,0.7,16)
        bStalCentreConnectorList = FromListConnector(bStalCentreConnectors) 
        self.nealProjection(self.hEdgeTopCells,self.visionFactCells,
                            bStalCentreConnectorList,'excitatory')
        bStalCentreConnectors = self.connectV1FeatureToObject(featWt,0.3,0.7,16)
        bStalCentreConnectorList = FromListConnector(bStalCentreConnectors) 
        self.nealProjection(self.sEdgeBottomCells,self.visionFactCells,
                            bStalCentreConnectorList,'excitatory')


        #redstalLeft
        rStalLeftConnectors = self.connectV1FeatureToObject(featWt,0,0.3,2)
        rStalLeftConnectorList = FromListConnector(rStalLeftConnectors) 
        self.nealProjection(self.hEdgeTopCells,self.visionFactCells,
                            rStalLeftConnectorList,'excitatory')
        rStalLeftConnectors = self.connectV1FeatureToObject(featWt,0,0.4,2)
        rStalLeftConnectorList = FromListConnector(rStalLeftConnectors) 
        self.nealProjection(self.sEdgeBottomCells,self.visionFactCells,
                            rStalLeftConnectorList,'excitatory')

        #red Stalactite Centre
        rStalCentreConnectors = self.connectV1FeatureToObject(featWt,0.3,0.7,5)
        rStalCentreConnectorList = FromListConnector(rStalCentreConnectors) 
        self.nealProjection(self.hEdgeTopCells,self.visionFactCells,
                            rStalCentreConnectorList,'excitatory')
        rStalCentreConnectors = self.connectV1FeatureToObject(featWt,0.4,0.8,5)
        rStalCentreConnectorList = FromListConnector(rStalCentreConnectors) 
        self.nealProjection(self.sEdgeBottomCells,self.visionFactCells,
                            rStalCentreConnectorList,'excitatory')

        #red stalRight
        rStalRightConnectors = self.connectV1FeatureToObject(featWt,0.7,1.0,3)
        rStalRightConnectorList = FromListConnector(rStalRightConnectors) 
        self.nealProjection(self.hEdgeTopCells,self.visionFactCells,
                            rStalRightConnectorList,'excitatory')
        rStalRightConnectors = self.connectV1FeatureToObject(featWt,0.8,1.0,3)
        rStalRightConnectorList = FromListConnector(rStalRightConnectors) 
        self.nealProjection(self.sEdgeBottomCells,self.visionFactCells,
                            rStalRightConnectorList,'excitatory')

        #doorWt = 2.0
        doorWt = 3.5

        #endDoorCentre
        #endDoorConnectors = self.connectV1FeatureToObject(2.0,0.7,1.0,7)
        #endDoorConnectorList = FromListConnector(endDoorConnectors) 
        #self.nealProjection(self.vEdgeLeftCells,self.visionFactCells,
        #                    endDoorConnectorList,'excitatory')
        endDoorConnectors = self.connectV1FeatureToObject(2.0,0.0,0.3,7)
        endDoorRightConnectorList = FromListConnector(endDoorConnectors) 
        #self.nealProjection(self.vEdgeRightCells,self.visionFactCells,
        #                    endDoorConnectorList,'excitatory')

        #doorLeft
        doorLeftConnectors = self.connectV1FeatureToObject(doorWt,0,0.3,6)
        doorLeftConnectorList = FromListConnector(doorLeftConnectors) 
        self.nealProjection(self.vEdgeLeftCells,self.visionFactCells,
                            doorLeftConnectorList,'excitatory')
        doorRightConnectors = self.connectV1FeatureToObject(doorWt,0,0.5,6)
        doorRightConnectorList = FromListConnector(doorRightConnectors) 
        self.nealProjection(self.vEdgeRightCells,self.visionFactCells,
                            doorRightConnectorList,'excitatory')


        #doorCentre
        doorCentreConnectors = self.connectV1FeatureToObject(doorWt,0.3,0.5,8)
        doorCentreConnectorList = FromListConnector(doorCentreConnectors) 
        self.nealProjection(self.vEdgeLeftCells,self.visionFactCells,
                            doorCentreConnectorList,'excitatory')
        #doorCentreConnectors = self.connectV1FeatureToObject(doorWt,0.5,0.8,8)
        doorCentreConnectors = self.connectV1FeatureToObject(doorWt,0.3,0.7,8)
        doorCentreConnectorList = FromListConnector(doorCentreConnectors) 
        self.nealProjection(self.vEdgeRightCells,self.visionFactCells,
                            doorCentreConnectorList,'excitatory')
        #doorRight
        doorLeftConnectors = self.connectV1FeatureToObject(doorWt,0.5,1.0,9)
        doorLeftConnectorList = FromListConnector(doorLeftConnectors) 
        self.nealProjection(self.vEdgeLeftCells,self.visionFactCells,
                            doorLeftConnectorList,'excitatory')
        doorRightConnectors = self.connectV1FeatureToObject(doorWt,0.7,1.0,9)
        doorRightConnectorList = FromListConnector(doorRightConnectors) 
        self.nealProjection(self.vEdgeRightCells,self.visionFactCells,
                            doorRightConnectorList,'excitatory')


    def factPrimesFact(self,fromFact,toFact):
        synWeight = 0.5
        connectors = []
        for fromOffset in range (0,5):
            for toOffset in range (0,5):
                fromNeuron = fromOffset+fromFact*5
                toNeuron = toOffset+toFact*5
                connectors=connectors+[(fromNeuron,toNeuron,synWeight,DELAY)]
        connectorList = FromListConnector(connectors) 
        self.nealProjection(self.visionFactCells,self.visionFactCells,
                            connectorList,'excitatory')


    def factStopsFact(self,fromFact,toFact):
        synWeight = -10.0
        connectors = []
        for fromOffset in range (0,5):
            for toOffset in range (0,5):
                fromNeuron = fromOffset+fromFact*5
                toNeuron = toOffset+toFact*5
                connectors=connectors+[(fromNeuron,toNeuron,synWeight,DELAY)]

        connectorList = FromListConnector(connectors) 
        self.nealProjection(self.visionFactCells,self.visionFactCells,
                            connectorList,'inhibitory')

    def factSlowsFact(self,wt, fromFact,toFact):
        synWeight = wt
        connectors = []
        for fromOffset in range (0,5):
            for toOffset in range (0,5):
                fromNeuron = fromOffset+fromFact*5
                toNeuron = toOffset+toFact*5
                connectors=connectors+[(fromNeuron,toNeuron,synWeight,DELAY)]

        connectorList = FromListConnector(connectors) 
        self.nealProjection(self.visionFactCells,self.visionFactCells,
                            connectorList,'inhibitory')

    def makeObjRecSynapses(self):
        #blue primes blue left,right and centre pyramid
        #self.factPrimesFact(11,0)
        #self.factPrimesFact(11,4)

        #doors slow each other
        self.factSlowsFact(-4.0,6,8)
        #self.factSlowsFact(-5.0,6,9)
        #self.factSlowsFact(-2.0,7,6)
        #self.factSlowsFact(-1.0,7,9)
        self.factSlowsFact(-2.0,8,9)
        #self.factSlowsFact(-2.0,8,6)
        #self.factSlowsFact(-2.0,9,8)
        #self.factSlowsFact(-2.0,9,7)
        #self.factSlowsFact(-0.5,9,6)

        #blue centre pyramid stops bc stalactite and vice-versa
        self.factStopsFact(4,16)
        self.factStopsFact(16,4)

        #blue left pyramid stops bc stalactite and vice-versa
        self.factStopsFact(0,16)
        self.factStopsFact(16,0)

        #blue stops left,centre and right door
        #self.factStopsFact(11,7)
        #self.factStopsFact(11,8)
        #self.factStopsFact(11,9)
        return


    def testNetNumber(self):
        synWeight = 1.0
        connectors = []
        for inputNeuron in range (0,900):
            connectors=connectors+[(inputNeuron,256,synWeight,DELAY)]
        connectorList = FromListConnector(connectors) 
        self.nealProjection(self.inputCells,self.nineOffCells,
                            connectorList,'excitatory')
        
    def makeVisionSynapses(self):
        #self.testNetNumber()
        self.makeInputToRetSynapses()
        self.makeRetToV1Synapses()
        #self.makeColourToFactSynapses()
        self.makeColourToObjFactSynapses()
        self.makeV1ToObjRecSynapses()
        self.makeObjRecSynapses()
        
    #---input functions
    def setInput(self,imagePixels,inputSource):
        weight = 20.0
        connectors = []
        cConnectors = []

        for onPixel in imagePixels:
            connectors = connectors + [(0,onPixel,weight,DELAY)]

        #print connectors
        excConnector = FromListConnector(connectors)
        self.nealProjection(inputSource,self.inputCells,excConnector,'excitatory')
        return connectors

    #---- recording functions
    def setVisionRecording(self):
        if self.simulator_name == 'spiNNaker':
            self.threeOnCells.record()
            self.threeOffCells.record()
            self.sixOnCells.record()
            self.sixOffCells.record()
            self.nineOnCells.record()
            self.nineOffCells.record()
            self.hEdgeBottomCells.record()
            self.hEdgeTopCells.record()
            self.vEdgeLeftCells.record()
            self.vEdgeRightCells.record()
            self.sEdgeBottomCells.record()
            self.sEdgeTopCells.record()
            self.orAngleCells.record()
            self.visionFactCells.record()
        elif self.simulator_name == 'nest':
            self.threeOnCells.record('spikes')
            self.threeOffCells.record('spikes')
            self.sixOnCells.record('spikes')
            self.sixOffCells.record('spikes')
            self.nineOnCells.record('spikes')
            self.nineOffCells.record('spikes')
            self.hEdgeBottomCells.record('spikes')
            self.hEdgeTopCells.record('spikes')
            self.pyramidLeftCells.record('spikes')
            self.pyramidRightCells.record('spikes')
            self.stalactiteLeftCells.record('spikes')
            self.stalactiteRightCells.record('spikes')


    #-cover functions to return internal values
    def getBWVisionInputCells(self):
        return self.bwVisualInputSource

    def getColourVisionInputCells(self):
        return self.colourVisualInputSource

    def getVisionFactCells(self):
        return self.visionFactCells

    def getLeftPyramid(self):
        return self.pyramidLeftCells

    def getRightPyramid(self):
        return self.pyramidRightCells

    def getLeftStalactite(self):
        return self.stalactiteLeftCells

    def getRightStalactite(self):
        return self.stalactiteRightCells


